import pygame
import physics
import world
import pymunk

class DynObj():
    def __init__(self, img,deadImg,glowImg, isDead, position, theWorld, minsize, maxsize, startmin = 1):
        #player should be able to pass through dead objects
        self.isDead = isDead
        self.img = img
        #store unscaled image for fast/correct scaling
        self.img_unscaled = img
        
        self.deadImg = deadImg
        self.deadImg_unscaled = deadImg
        self.rect= img.get_rect()
        self.rect.topleft = position
        self.physicsObject = theWorld.physicsSystem.addStaticBox(theWorld.physicsSystem.space, self.rect)
        
        self.minsize = minsize
        self.maxsize = maxsize
        
        self.img.set_colorkey(pygame.Color(0,0,0,0))
        self.glowing = glowImg
        self.glowing_unscaled = self.glowing.copy()
        
        if(startmin):
            self.rect.width = abs(minsize[0])
            self.rect.height = abs(minsize[1])
        else:
            self.rect.width = abs(maxsize[0])
            self.rect.height = abs(maxsize[1])
            if(maxsize[0]<0):
                self.rect.topleft = (self.rect.topleft[0] + maxsize[0], self.rect.topleft[1])
            if(maxsize[1]<0):
                self.rect.topleft = (self.rect.topleft[0], self.rect.topleft[1] + maxsize[1] +32)
            
        
        self.startPosition = position
        
        self.grow(0,0,theWorld)

    # Does this object collide with the given rectangle?
    def collides(self, rect):
        return pygame.Rect.colliderect(self.rect, rect)
        

    def draw(self, screen,cameraPosition):
        transformed = pygame.Rect(self.rect.topleft[0] - cameraPosition[0], self.rect.topleft[1] - cameraPosition[1], self.rect.width, self.rect.height)
        
        if self.isDead :
            if(self.rect.collidepoint(pygame.mouse.get_pos()[0] + cameraPosition[0],pygame.mouse.get_pos()[1]+ cameraPosition[1])):
                screen.blit(self.deadImg,transformed)
            else:
                screen.blit(self.deadImg,transformed)
        else :
            if(self.rect.collidepoint(pygame.mouse.get_pos()[0] + cameraPosition[0],pygame.mouse.get_pos()[1]+ cameraPosition[1])):
                screen.blit(self.glowing,transformed)
            else:
                screen.blit(self.img,transformed)
                
        
            
    def die(self,theWorld):
        if(self.isDead):
            return
        
        self.isDead = 1
     
        theWorld.physicsSystem.space.remove_static([self.physicsObject[1]])
        self.shrink(theWorld)
   
    
    def shrink(self, theWorld): 
        self.rect.height = self.minsize[1]
        self.rect.width = self.minsize[0]
        self.rect.topleft = self.startPosition
        
        self.grow(0,0,theWorld,1, 0)
    
    def shrink2(self,dx,dy,theWorld):
        if(dx <=0):
            self.rect.width+=dx
        # If growth sizes are negative, move the rect positively and grow it negatively.
        else:
            self.rect.topleft = (self.rect.topleft[0]+dx,self.rect.topleft[1])
            self.rect.width = max(self.rect.width-dx, 32)
        if(dy <=0):
            self.rect.height+=dy
        else:
            self.rect.topleft =(self.rect.topleft[0],self.rect.topleft[1]+dy)
            self.rect.height =max(self.rect.height-dy,32)
   
             
        # Scale up/down images.
        scaled1=pygame.transform.smoothscale(self.img_unscaled, (self.rect.width ,self.rect.height))
        self.img = scaled1
        
        scaled2 = pygame.transform.smoothscale(self.deadImg_unscaled, (self.rect.width,self.rect.height))
        self.deadImg = scaled2

   
    # Causes the dynobj to grow towards its maximum size
    def regenerate(self,theWorld):
        # Amount to grow.
        dx = 0
        dy = 0
        wasPreviouslyDead = 0
        if(self.isDead):
            wasPreviouslyDead = 1
        
        self.isDead = 0
        
        # If the rect is smaller than the max size, grow positively
        if(self.rect.width<self.maxsize[0]):
            dx = 2
        #Otherwise, if the rect is bigger, and its absolute size is less than the negation of the maximum size,
        # grow negatively
        elif(self.rect.width>self.maxsize[0] and (self.rect.width< -1*self.maxsize[0])): 
            dx = -2

            
        if(self.rect.height<self.maxsize[1]):
            dy = 2
        elif(self.rect.height>self.maxsize[1] and (self.rect.height < -1*self.maxsize[1])):
            dy = -2
            
        self.grow(dx,dy,theWorld,wasPreviouslyDead)
   
         
    # Grow the rect (supports negative values). Creates and destroys physics objects.
    def grow(self, dx, dy, theWorld, wasPreviouslyDead = 0, makeNewPhysicsShape = 1):

        if(dx >=0):
            self.rect.width+=dx
        # If growth sizes are negative, move the rect positively and grow it negatively.
        else:
            self.rect.topleft = (self.rect.topleft[0]+dx,self.rect.topleft[1])
            self.rect.width-=dx
        if(dy >=0):
            self.rect.height+=dy
        else:
            self.rect.topleft =(self.rect.topleft[0],self.rect.topleft[1]+dy)
            self.rect.height-=dy
            
        # Scale up/down images.
        scaled1=pygame.transform.smoothscale(self.img_unscaled, (self.rect.width + dx,self.rect.height+dy))
        self.img = scaled1
        
        scaled2 = pygame.transform.smoothscale(self.deadImg_unscaled, (self.rect.width+dx,self.rect.height+dy))
        self.deadImg = scaled2

        scaled3 = pygame.transform.smoothscale(self.glowing_unscaled, (self.rect.width+dx,self.rect.height+dy))
        self.glowing = scaled3

        # Destroy old physics and make a new one.
        if(not wasPreviouslyDead):
            theWorld.physicsSystem.space.remove_static([self.physicsObject[1]])
        if(makeNewPhysicsShape):
            self.physicsObject = theWorld.physicsSystem.addStaticBox(theWorld.physicsSystem.space, self.rect)
        
    def update(self, theWorld):
        if(self.isDead):
            self.shrink(theWorld)
            