import pygame
import pymunk
# Wrapper around pymunk space.
class Physics():
    
    def __init__(self, space):
        self.space = space
        self.clock = pygame.time.Clock()
        
    def update(self):
        # Assume 60 FPS
        self.space.step(1 / 60.0)
        self.clock.tick(60)
    
    def addSpike(self, position):
        mass = 1
        radius = 5
        
        body = pymunk.Body(mass, 999)
        body.position = (position[0], 480 - position[1])
        
        shape = pymunk.Circle(body, radius)
        self.space.add(body,shape)
        
        return (body,shape)
        
    # Convienience method for creating a ball.
    # Position is in pygame coordinates.
    # It is translated to pymunk coords.
    def addBall(self, theMass, theRadius, position):
        mass = theMass
        radius = theRadius
        # Hack to make player not roll around.
        inertia = 999999999
        
        body = pymunk.Body(mass, inertia)
        #Conversaion to pymunk coords
        body.position = (position[0], 480 - position[1])
        
        shape = pymunk.Circle(body, radius)
        shape.friction = 0.8
        self.space.add(body, shape) 
        return (body, shape)
   
    # Convenience method for adding a static box around a rect..
    # Converts pygame coords to pymunk coords
    def addStaticBox(self, space, rect):
        body = pymunk.Body(pymunk.inf, pymunk.inf) # 1

        # Conversion
        body.position = (rect.center[0], 480-rect.center[1])
        
        # Get relative positions of each of the rect's verticies
        t_topLeft     = (rect.topleft[0]     - rect.center[0] ,rect.topleft[1]     - rect.center[1] )
        t_topRight    = (rect.topright[0]    - rect.center[0] ,rect.topright[1]    - rect.center[1] )
        t_bottomLeft  = (rect.bottomleft[0]  - rect.center[0] ,rect.bottomleft[1]  - rect.center[1] )
        t_bottomRight = (rect.bottomright[0] - rect.center[0] ,rect.bottomright[1] - rect.center[1] )
        
     
        # Polygon expects a list of verticies
        verticies = [t_bottomRight, t_bottomLeft, t_topLeft, t_topRight]
        polygon = pymunk.Poly(body, verticies)
        polygon.friction = 0.5
        space.add_static(polygon)
        return (body,polygon)
    
    
        