# Simple Controller class for moving te camera
class PIDController():
    def __init__(self, p,i,d):
        self.p = p
        self.i = i
        self.d = d
        self.lastError = [0,0]
        self.sumError = [0,0]

    # How much should the controller move the camera?
    # Offset is simply added onto position.
    def getOutput(self, position, target, offset, threshold):
        
        # Vector from position to target
        error = (target[0] - (position[0] + offset[0]), target[1] - (position[1] + offset[1]))
        
        # Don't move if square of distance is less than threshold
        if(error[0]*error[0] + error[1]*error[1]) < threshold:
            return (0,0)
        
        # Derivative of the error term (does not take dt into acount)
        dError = (error[0] - self.lastError[0], error[1] - self.lastError[1])
        
        # Update sums and derivatives
        self.sumError[0]+= error[0]
        self.sumError[1]+=error[1]
        self.lastError = error
        
        output = [0,0]
        
        # PID equation
        output[0] = self.p*error[0] + self.i*self.sumError[0] + self.d*dError[0]
        output[1] = self.p*error[1] + self.i*self.sumError[1] + self.d*dError[1]
        
        return output
    
    # Conveinience method that alters camera position in-place
    def trackPlayer(self, cameraPosition, playerPosition):
        output = self.getOutput(cameraPosition, playerPosition, (640/2, 480/2), 0)
        cameraPosition[0]+=output[0]
        cameraPosition[1]+=output[1]
        
    