import pygame
import tile
import world

import pymunk

class Player():
    def __init__(self, img, (body,shape)):
        self.img = img
        self.rect = img.get_rect();
        self.rect.topleft = body.position
        self.position = body.position
        self.body = body
        self.shape = shape
        self.touchingFloor = 1
        
    def update(self, theWorld):
        self.position = [self.body.position[0], 480-self.body.position[1]]            
        self.rect.center = (self.position[0],self.position[1]-16)
        playerPosition = (int(self.position[0]/32), int(self.position[1]/32))
        tileBelow = theWorld.getTile(playerPosition[1]+1, playerPosition[0])
        
        if((None is tileBelow or tileBelow.empty or tileBelow.passable)and not(self.touchingDynamicObject(theWorld))):
            self.touchingFloor = 0
        else:
            self.touchingFloor = 1
        
    
    def left(self):
        if self.touchingFloor:
            if self.body.velocity[0] > -200:
                self.body.apply_impulse((-50,0))
        else:
            if self.body.velocity[0] > -200:
                self.body.apply_impulse((-40,0))
        
    def right(self):
        if self.touchingFloor:
            if self.body.velocity[0] < 200:
                self.body.apply_impulse((50,0))
        else:
            if self.body.velocity[0] < 200:
                self.body.apply_impulse((40,0))
    
    def jump(self):
        if(self.touchingFloor):
            self.body.apply_impulse((0,350))
        
    def draw(self, screen, cameraPosition):
        #Offset by camera position
        transformed = pygame.Rect(self.rect.topleft[0] - cameraPosition[0], self.rect.topleft[1] - cameraPosition[1], self.rect.width, self.rect.height)
        
        toDraw = self.img
        
        if(self.body.velocity[0] < 0):
            toDraw = pygame.transform.flip(self.img, 1, 0)
        screen.blit(toDraw,transformed)
    

        
    def collidesRect(self, rect):
        return pygame.Rect.colllideRect(rect)
    
    def collidesTile(self, tile):
        return tile.collides(self.rect)
    
    def collidesDynObj(self, object):
        return object.collides(self.rect)
    
    def touchingDynamicObject(self, theWorld):
        bigRect = pygame.Rect(self.rect)
        bigRect.topleft = (bigRect.topleft[0] + 10, bigRect.topleft[1]+64)
        bigRect.width = 2
        bigRect.height = 15
        
        for obj in theWorld.dynObjs:
            if ((obj.rect.colliderect(bigRect)) and not obj.isDead):
                return 1
            
        return 0
    