import pygame
import player
import random
from world import World

playerColor = pygame.Color(0,0,255)
upwards = pygame.Color(34,177,76)
downwards = pygame.Color(255,242,0)
leftwards = pygame.Color(255,126,0)
rightwards = pygame.Color(0,183,239)
cross = pygame.Color(111,49,152)
growcolor = pygame.Color(139,208,130)
deadcolor = pygame.Color(237,28,36)
wincolor = pygame.Color(255,163,177)
spikelaunchcolor = pygame.Color(180,180,180)



tileimages =[]

for n in range(1,51):
    blockstring = "block"
    if(n<10):
        blockstring+="0"
    if(n==9):
        continue
    blockstring += str(n)
    blockstring += ".png"
    tileimages.append(pygame.image.load(blockstring))
    
staticDict = {0:tileimages, 0xabcdef:[pygame.image.load("black.png")]}
    
    
def isPlayerColor(col):
    return (col.r == playerColor.r and col.g == playerColor.g and col.b == playerColor.b)
def readmap(img, numRows, numCols, physicsSystem):
    world = World(numCols, numRows, physicsSystem)
    string = pygame.image.tostring(img,"ARGB")
    mapnum = map(ord, string)
    playerList = []
    
    for x in range(numCols):
        for y in range(numRows):
            color = img.get_at((y,x))
            if(color == wincolor):
                world.addWinRect(y, x)
            elif(color == deadcolor):
                world.addDeathRect(y, x)  
            elif(color == spikelaunchcolor):
                world.addLauncher(y, x)  
    for x in range(numCols):
        for y in range(numRows):
            color = img.get_at((y,x))
            colorNum = color.b+(color.g<<8)+(color.r<<16)
            if colorNum in staticDict:
                index = random.randint(0,48)
                if(colorNum == 0xabcdef):
                    index = 0
                    
                world.addTile(x, y, colorNum, staticDict[colorNum][index])
            elif isPlayerColor(color):
                thePlayer = player.Player(pygame.image.load("druid.png"),world.physicsSystem.addBall(5, 16, (y*32,x*32)))
                playerList.append(thePlayer)
    minsize = (32,32)
    plantimg = pygame.image.load("living_vine.png")
    deadImg = pygame.image.load("dead_vine.png")
    glowImg = pygame.image.load("glowing_vine.png")
    isDead = 0
    grow = pygame.Color(0,0,0)
    
    for row in range(numRows):
        for col in range(numCols):
            if img.get_at((row, col)) == downwards:
                index = col
                position = (row * 32, col * 32)
                color = get_at_2(img,(row,col+1))
                grow = get_at_2(img,(row,col+1))
                while index < numCols and (color == grow or color == cross):
                    index += 1
                    color = get_at_2(img,(row,index))
                maxsize = (32,32 * (index - col))
                print("RIGHT PLANT MAX SIZE:",index- col)
                startgrown = (grow==growcolor)
                obj = world.addDynamicObject(pygame.transform.rotate(plantimg,-90), pygame.transform.rotate(deadImg,-90),pygame.transform.rotate(glowImg,-90), isDead, position, minsize, maxsize, not startgrown)

            if img.get_at((row, col)) == upwards:
                index = col
                position = (row * 32, col * 32)
                color = get_at_2(img,(row,index-1))
                grow = get_at_2(img,(row,index-1))
                while index > 0 and (color == grow or color == cross):
                    index -= 1
                    color = get_at_2(img,(row,index))
                    
                print ("LEFT PLANT MAX SIZE:", index-col)
                maxsize = (32,32 * (index - col))
                startgrown = (grow==growcolor)
                obj = world.addDynamicObject(pygame.transform.rotate(plantimg,90), pygame.transform.rotate(deadImg,90),pygame.transform.rotate(glowImg,90), isDead, position, minsize, maxsize, not startgrown)

    for col in range(numCols):
        for row in range(numRows):
            if img.get_at((row, col)) == rightwards:
                index = row
                position = (row * 32, col * 32)
                color = get_at_2(img,(row+1,col))
                grow = get_at_2(img,(row+1,col))
                while index < numRows and (color == grow or color == cross):
                    index += 1
                    color = get_at_2(img,(index,col))
                    print (color,grow,cross)
                print ("DOWN PLANT MAX SIZE: ", index-row)
                maxsize = (32 * (index - row),32)
                print maxsize
                
                startgrown = (grow==growcolor)
                obj = world.addDynamicObject(plantimg, deadImg,glowImg, isDead, position, minsize, maxsize, not startgrown)

            if img.get_at((row, col)) == leftwards:
                index = row
                position = (row * 32, col * 32)
                color = get_at_2(img,(row-1,col))
                grow = get_at_2(img,(row-1,col))
                while index > 0 and (color == grow or color == cross):
                    index -= 1
                    color = get_at_2(img,(index,col))
                print ("UP PLANT MAX SIZE: ", index-row)
                maxsize = (32 * (index - row),32)
                startgrown = (grow==growcolor)
                obj = world.addDynamicObject(pygame.transform.rotate(plantimg,180), pygame.transform.rotate(deadImg,180),pygame.transform.rotate(glowImg,180), isDead, position, minsize, maxsize, not startgrown)

    return (world, playerList[0])

def get_at_2(img, (x,y)):
    return img.get_at((x,y))
