import pygame
import pymunk

# Simple class for representing a tile.
# Tiles can be passable, or non-passable.
class Tile():
    
    #Constructor
    def __init__(self, id, img, position,space, passable = 0, empty = 0):
        self.image = img
        self.rect = img.get_rect()
        # 1 or 0
        self.passable = passable
        self.rect.topleft = position
        # For storing "types" of tiles.
        # Not sure if we should just use this
        self.id = id
        
        self.empty = empty
        
        if(not empty and not passable):
            (body, shape) = self.addStaticBox(space, self.rect)
            self.body = body
            self.shape = shape
    
    def addStaticBox(self, physicsSystem, rect):
        body = pymunk.Body(pymunk.inf, pymunk.inf) # 1

        body.position = (rect.center[0], 480-rect.center[1])
        
        t_topLeft = (rect.topleft[0] - rect.center[0],rect.topleft[1] - rect.center[1] )
        t_topRight = (rect.topright[0] - rect.center[0],rect.topright[1] - rect.center[1] )
        t_bottomLeft = (rect.bottomleft[0] - rect.center[0],rect.bottomleft[1] - rect.center[1] )
        t_bottomRight = (rect.bottomright[0] - rect.center[0],rect.bottomright[1] - rect.center[1] )
        
     
        
        verticies = [t_bottomRight, t_bottomLeft, t_topLeft, t_topRight]
        polygon = pymunk.Poly(body, verticies)
        polygon.friction = 0.5
        physicsSystem.space.add_static(polygon)
        return (body,polygon)
        
    # Draws image with respect to the camera position
    def draw(self, screen, cameraPosition):
        if(self.empty):
            return
        #Offset by camera position
        transformed = pygame.Rect(self.rect.topleft[0] - cameraPosition[0], self.rect.topleft[1] - cameraPosition[1], self.rect.width, self.rect.height)
        screen.blit(self.image,transformed)
    
    # Does this tile collie with the given rectangle?
    def collides(self, rect):
        if self.passable:
            return 0
        return pygame.Rect.colliderect(self.rect, rect)
        