import pygame
import tile
import dynObj
import pymunk
import physics
import spikelauncher
import random
pygame.mixer.init()
mindbeamSound = pygame.mixer.Sound("mindbeam.wav")
mindbeamchannel = mindbeamSound.play()
mindbeamSound.stop()
mindbeamchannel.set_volume(0.4)
plantdiesound = pygame.mixer.Sound("plantdie.wav")
plantdiechannel = plantdiesound.play()
plantdiechannel.set_volume(0.15)
plantdiesound.stop()

gradientimage = pygame.image.load("gradient.png")

acids = [pygame.image.load("acid_top01.png"),pygame.image.load("acid_top02.png"),pygame.image.load("acid_top03.png"),pygame.image.load("acid_top04.png")]

    
# Class for representing the game world and its tile map
class World():
    
    # Constructor taking the number of rows and columns
    def __init__(self, numRows, numCols, physicsSystem):
        #List of Tiles
        self.tilemap = []

        self.physicsSystem = physicsSystem
        img = pygame.image.load("tile.png")
        for r in range(0, numRows):
            tilerow = []
            for c in range(0,numCols):
                tilerow.append(tile.Tile(0, img, self.getTileCoordinates(r,c,32),self.physicsSystem,1, 1))
            self.tilemap.append(tilerow)
        
        #Dimensions of the tile map
        self.dim = (numRows, numCols)
        self.dynObjs = []
    
        self.winningRects = []
        self.deathRects = []
        self.deathStarts = []
        
        self.launchers = []
        
        self.clipRect = pygame.Rect(0,0,32*numCols, 32*numRows)
    
        self.acidTimer = 0
        
    def wins(self, position):
        for rectangle in self.winningRects:
            if rectangle.collidepoint(position):
                return 1
        return 0
    
    def dies(self,position):
        for rectangle in self.deathRects:
            if rectangle.collidepoint(position):
                return 1
        return 0
    
    def addWinRect(self, row, col):
        self.winningRects.append(pygame.rect.Rect(32*row, 32*col, 32,32))
    
    def addDeathRect(self, row, col):
        self.deathRects.append(pygame.rect.Rect(32*row, 32*col, 32,32))
        self.deathStarts.append(random.randint(0,3))
    
    def addDynamicObject(self,img,deadImg,glowImg, isDead, position, minsize, maxsize, startmin = 1):
        dyn = dynObj.DynObj(img, deadImg,glowImg, isDead, position,self, minsize, maxsize, startmin)
        dyn.maxtiles = [(position[0]/32, position[1]/32)]
        self.dynObjs.append(dyn)
        return dyn
    
    #Insert a tile at the given row, and column, with the given id, and image.
    def addTile(self, row, col, id, img, passable = 0):
        imgrect = img.get_rect();
        self.tilemap[row][col] = tile.Tile(id, img, self.getTileCoordinates(row, col, imgrect.width),self.physicsSystem, passable)
    
    #Get tile at given row, column  
    def getTile(self, row, col):
        if(row<0 or col<0 or row>=self.dim[0] or col>=self.dim[1]):
            return None
        else:
            return self.tilemap[row][col]
    
    def update(self):
        for obj in self.dynObjs:
            obj.update(self)
            
        self.physicsSystem.update();
            
    #Renders each tile with the offset of the camera in mind
    def draw(self, screen, cameraPosition, thePlayer):
        for tilerow in self.tilemap:
            for tile in tilerow:
                tile.draw(screen, cameraPosition)
        
        for thing in self.dynObjs:
            thing.draw(screen,cameraPosition)
         
        for gradient in self.winningRects:
            transformed = pygame.Rect(gradient.left-cameraPosition[0], gradient.top - cameraPosition[1],32,32)
            screen.blit(gradientimage, transformed) 
        acidindex = 0
        for acid in self.deathRects:
            
            transformed = pygame.Rect(acid.left-cameraPosition[0], acid.top - cameraPosition[1],32,32)
            screen.blit(acids[((self.acidTimer/(100))%4 + self.deathStarts[acidindex])%4], transformed)   
            self.acidTimer+=1
            acidindex+=1
        shouldDie = 0
        for launcher in self.launchers:
            shouldDie = (shouldDie or launcher.update(self, screen, cameraPosition, thePlayer.rect))
            
        return shouldDie
            
    # Gets the top left pixel of the given tile
    def getTileCoordinates(self, row, col, tilewidth):
        return (col*tilewidth, row*tilewidth)

    def kill(self,x,y):
         for obj in self.dynObjs:
            if obj.rect.collidepoint(x,y):
                obj.die(self)
                if(not plantdiechannel.get_busy()):
                    plantdiechannel.queue(plantdiesound)
                    plantdiechannel.set_volume(0.1)
    def regenerate(self, x, y):
        for obj in self.dynObjs:
            if obj.rect.collidepoint(x,y):
                obj.regenerate(self)
                if(not mindbeamchannel.get_busy() ):
                    mindbeamchannel.queue(mindbeamSound)
                    mindbeamchannel.set_volume(0.1)
    
    
    def addLauncher(self,r,c):
        launcher = spikelauncher.SpikeLauncher(self, r, c, pygame.image.load("flower01.png"))
        self.launchers.append(launcher)
        
    def collidesWithAnythingAtAll(self, rectangle, playerRect):
        rectToCheck = rectangle.inflate(2,2)
        for obj in self.dynObjs:
            if(obj.rect.colliderect(rectToCheck )):
                return 1
        
        if(playerRect.colliderect(rectToCheck )):
            return 2
        
        (r,c) = (int(rectToCheck .topleft[0]/32), int(rectToCheck .topleft[1]/32))
        
        for row in self.tilemap:
            for t in row:
                if(t.rect.colliderect(rectToCheck) and not t.passable and not t.empty):
                    return 3
                
        if(rectToCheck.topleft[0]<0 or rectToCheck.topleft[0]>2500 or rectToCheck.topleft[1]>1500 or rectToCheck.topleft[1]<0):
            return 4

        
        
        return 0